#------------------------------------------------------------------------
#
#  IBM Software
#  Licensed Material - Property of IBM
#  (C) Copyright International Business Machines Corp. 2010, 2017
#  All Rights Reserved.
#
#  U.S. Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#------------------------------------------------------------------------
#  kips.tcl
#------------------------------------------------------------------------
#
#  Generate periodic performance messages similar to the standard mambo
#  KIPS messages, but on an interval determined by the user.
#
#------------------------------------------------------------------------

namespace eval kips {
    variable version       "1.0"
    variable interval      1000000000
    variable start_time    0
    variable start_instr   0
    variable next_trig     0
}

proc kips::report_kips { args } {
    variable interval
    variable start_time
    variable start_instr
    variable next_trig

    #puts "mykips::report called with args = $args"
    set sim mysim

    set elapsed_instr [expr [$sim display instruction_count] - $start_instr]
    set elapsed_time [expr [clock seconds] - $start_time]
    set minstr [expr $elapsed_instr.0/1e6]
    set mips [expr $minstr/$elapsed_time]

    puts [format "[$sim display cycles]: [$sim display instruction_count]: Executed %4.2fM instructions : %4.2f Mega-Inst/Sec" $minstr $mips]

    set start_time [clock seconds]
    set start_instr [$sim display instruction_count]
    set next_trig [expr [$sim display cycle]+$interval]

    $sim trigger set cycle $next_trig kips::report_kips
}

proc kips::start { { opt_interval 1000000000 } { sim mysim } } {
    variable interval
    variable start_time
    variable start_instr
    variable next_trig

    set interval [expr int($opt_interval)]
    set start_time [clock seconds]
    set start_instr [$sim display instruction_count]
    set next_trig [expr int( floor(([$sim display cycle]+$interval-1)/$interval) * $interval ) ]

    $sim trigger set cycle $next_trig kips::report_kips
}

proc kips::stop { { sim mysim } } {
    variable next_trig

    if {$next_trig > 0} {
        $sim trigger clear cycle $next_trig
    }
    set next_trig 0
}
