#####################################################
#####################################################
## Licensed Materials - Property of IBM.
## (C) Copyright IBM Corporation 2004, 2017
## All Rights Reserved.
##
## US Government Users Restricted Rights -
## Use, duplication or disclosure restricted by
## GSA ADP Schedule Contract with IBM Corporation.
##
## Filename   : osinfo.tcl
##
## Purpose    : TCL os-info events (built on bcast)
##
#####################################################
#####################################################

namespace eval OSInfo {
    variable version            "1.0"
    variable pid_map
    variable exec_map
    variable map
}

package provide OSInfo $OSInfo::version
package require BCast


####
#
proc OSInfo::List { {mct_list all} } {
    variable map

    if {[string equal $mct_list "all"]} {
        set mct_list [regsub -all {\S+,(\d:\d:\d)} [array names OSInfo::map] {\1}]
        set mct_list [lsort -unique $mct_list]
    }

    foreach i $mct_list {
        set record [regsub -all {,\d:\d:\d} [array get OSInfo::map "*,$i"] {}]
        puts "[list $i $record]"
    }
}

####
#
proc OSInfo::List_PID { {pid_list all} } {
    variable pid_map

    if {[string equal $pid_list "all"]} {
        set pid_list [lsort -integer [array names OSInfo::pid_map]]
    }

    foreach p $pid_list {
        if {[info exists OSInfo::pid_map($p)]} {
            puts "[list $p $OSInfo::pid_map($p)]"
        }
    }
}

####
#
proc OSInfo::List_Exec { {exec_list all} } {
    variable exec_map

    if {[string equal $exec_list "all"]} {
        set exec_list [lsort -dictionary [array names OSInfo::exec_map]]
    }

    foreach e $exec_list {
        if {[info exists OSInfo::exec_map($e)]} {
            puts "[list $e $OSInfo::exec_map($e)]"
        }
    }
}

####
#
proc OSInfo::Dispatch {args} {
    variable pid_map
    variable map

    if {[llength $args] == 1} {
        array set p [lindex $args 0]
    } else {
        array set p $args
    }

    if {![info exists p(mct)]} {
        return -code error "OSInfo::Dispatch -- missing mct parameter"
    }
    set mct $p(mct)

    # Create unmap args
    if {![info exists map(pid,$mct)]} {
        set map(pid,$mct) "none"
        set map(tid,$mct) "none"
        set map(exec,$mct) "none"
    }

    # PID
    if {[info exists p(pid)]} {
        set pid [format "%ld" $p(pid)]
        if {$pid != $map(pid,$mct)} {
            BCast::Announce pid $mct pid $pid
        }
        set map(pid,$mct) $pid
    } else {
        set pid $map(pid,$mct)
    }

    # TID
    if {[info exists p(tid)]} {
        set tid [format "%ld" $p(tid)]
        if {$tid != $map(tid,$mct)} {
            BCast::Announce tid $mct tid $tid
        }
        set map(tid,$mct) $tid
    }

    # EXEC
    if {[info exists p(exec)]} {
        set exec $p(exec)
        if {![string equal $exec $map(exec,$mct)]} {
            set pid_map($pid) $exec
            BCast::Announce exec $mct exec $exec
            #lappend exec_map($exec) $mct
            #set exec_map($exec) [lsort -unique $exec_map($exec)]
        }
        set map(exec,$mct) $exec
    } elseif {[info exists pid_map($pid)]} {
        set exec $pid_map($pid)
        if {![string equal $exec $map(exec,$mct)]} {
            BCast::Announce exec $mct exec $exec
        }
        set map(exec,$mct) $exec
    }
}
