#------------------------------------------------------------------------
#
#	IBM Software
#	Licensed Material - Property of	IBM
#	(C) Copyright International Business Machines Corp. 2020
#	All Rights Reserved.
#
#	U.S. Government	Users Restricted Rights - Use, duplication or
#	disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#------------------------------------------------------------------------
#  boot-linux-le-skiboot-p10.tcl
#
#  Boot Linux using real skiboot firmware.
#
#------------------------------------------------------------------------
# Run with e.g.:
#   /opt/ibm/systemsim-p10-<version>/run/p10/linux>../power10 -f boot-linux-le-skiboot-p10.tcl
#
# needs a ./skiboot.lid, a little endian ./vmlinux, and a little endian
# ./disk.img
#
#------------------------------------------------------------------------

#Images directory path - use /opt/ibm/systemsim-p10-<version>/examples/fetch_<>.sh scripts to download 
#   the images into this directory.  e.g. /opt/ibm/systemsim-p10-<version>/images>../examples/fetch_disk_image.sh 
set img_dir /opt/ibm/systemsim-p10-1.1-0/images

source $env(LIB_DIR)/p10/mambo_init.tcl

proc mconfig { name env_name def } {
    global mconf
    global env

    if { [info exists env($env_name)] } { set mconf($name) $env($env_name) }
    if { ![info exists mconf($name)] } { set mconf($name) $def }
}

# Location of application binary to load
mconfig boot_image SKIBOOT $img_dir/skiboot.lid

# Payload: Allow for a Linux style ramdisk/initrd
mconfig payload PAYLOAD $img_dir/vmlinux

# Linux commandline
mconfig linux_cmdline LINUX_CMDLINE "rw raid=noautodetect panic_timeout=-1 root=/dev/pmem0 root=/dev/pmem0"
#mconfig linux_cmdline LINUX_CMDLINE "rw raid=noautodetect panic_timeout=-1 root=/dev/pmem0 sshport=10975 mamboip=172.19.231.94 hostip=172.19.231.93 broadcastip=172.19.231.95"
mconfig pmem_disk PMEM_DISK $img_dir/disk.img

#
# Include various utilities
#

source $env(LIB_DIR)/common/epapr.tcl
if {![info exists of::encode_compat]} {
    source $env(LIB_DIR)/common/openfirmware_utils.tcl
}

proc config_hook { conf } {
   $conf config cpus 1
   $conf config processor/number_of_threads 1
   $conf config memory_size 4G
   $conf config machine_option/SIMGETTIME_IS_DETERMINISTIC TRUE
   $conf config processor_option/ATTN_STOP true
   #$conf config memory use_swap
}

source $env(RUN_DIR)/p10/p10-devtree-skiboot.tcl

proc clear_trigger_and_stop { sim mcm args } {
    array set triginfo $args
    set sim $triginfo(sim)
    $sim mcm $mcm trigger clear console $triginfo(match)
    simstop
}
proc wait_for_bash_prompt { { mcm 0 } { sim mysim } } {
    $sim mcm $mcm trigger set console "root@ubuntu1804mambo:~#" { clear_trigger_and_stop $sim $mcm}
    $sim go
}
proc wait_for_le_debian_login_console { {mcm 0 } { sim mysim } } {
    $sim trigger set console "ubuntu1804mambo login:" { clear_trigger_and_stop $sim 0}
    $sim go
}
proc wait_for_password_console { {mcm 0 } { sim mysim } } {
    $sim trigger set console "Password:" { clear_trigger_and_stop $sim 0}
    $sim go
}
puts "Booting with skiboot $mconf(boot_image)..."
puts "Booting with kernel $mconf(payload)..."
puts "Booting with PMEM $mconf(pmem_disk)..."

# Set run speed
mysim mode turbo

#bit accurate mode
#mysim set bit_accurate on

# use this snippet to do automated login, etc
wait_for_le_debian_login_console
console_input mysim "root"
wait_for_password_console
console_input mysim "mambo"

# use this snippet to begin workload execution, etc.
wait_for_bash_prompt
console_input mysim "cat /proc/cpuinfo"

mysim go

##### the real work goes here #####

